'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setup = exports.trigger = exports.getLocation = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _uuid = require('uuid');

var _frames = require('@r/frames');

var frames = _interopRequireWildcard(_frames);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var JAIL_ID = 'gtm-' + (0, _uuid.v4)();
var loaded = false;
var PENDING_EVENTS = [];

var getJail = function getJail() {
  var jail = document.getElementById(JAIL_ID);
  return jail && jail.contentWindow ? jail : null;
};

var queueEvent = function queueEvent(eventName, payload) {
  PENDING_EVENTS.push({ eventName: eventName, payload: payload });
};

var getLocation = exports.getLocation = function getLocation() {
  return {
    href: location.href,
    hostname: location.hostname,
    origin: location.origin,
    pathname: location.pathname || '/',
    search: location.search.replace(/^\?/, ''),
    hash: location.hash.replace(/^#/, '')
  };
};

var trigger = exports.trigger = function trigger(eventName, payload) {
  var jail = getJail();

  // Make sure GTM is loaded, it might not be ready since
  // we load it asyncronously.
  if (!jail || !loaded) {
    queueEvent(eventName, payload);
    return;
  }

  if (payload) {
    frames.postMessage(jail.contentWindow, 'data.gtm', payload);
  }

  frames.postMessage(jail.contentWindow, 'event.gtm', {
    event: eventName
  });
};

var onload = function onload() {
  var jail = getJail();

  if (!jail || !loaded) {
    // Not loaded yet.
    return;
  }

  // Explicitly copy the pending events to a new array and clear the queue.
  // `trigger` appends to PENDING_EVENTS so this is to prevent any issues
  // such as the queue growing forever (although that shouldn't happen)
  var pendingEvents = PENDING_EVENTS.slice();
  PENDING_EVENTS = [];

  pendingEvents.forEach(function (_ref) {
    var eventName = _ref.eventName,
        payload = _ref.payload;
    return trigger(eventName, payload);
  });
};

var setup = exports.setup = function setup(_ref2) {
  var containerId = _ref2.containerId,
      jailUrl = _ref2.jailUrl,
      payload = _ref2.payload;

  if (!containerId) {
    // eslint-disable-next-line no-console
    console.warn('GTM#setup: No `containerId` specified, skipping.');
    return;
  }

  if (!jailUrl) {
    // eslint-disable-next-line no-console
    console.warn('GTM#setup: No `jailUrl` specified, skipping.');
    return;
  }

  // Ensure events are queued until the jail is completely loaded.
  frames.listen('gtm');
  frames.receiveMessageOnce('loaded.gtm', function () {
    loaded = true;
    frames.stopListening('gtm');
  });

  var jail = document.createElement('iframe');
  jail.style.display = 'none';
  jail.id = JAIL_ID;
  jail.name = JSON.stringify(_extends({
    referrer: document.referrer
  }, getLocation(), payload));

  jail.src = jailUrl + '?id=' + containerId;
  jail.onload = onload;

  document.body.appendChild(jail);
};
